from flask import Blueprint, render_template, redirect, url_for, request, jsonify, g
from models.user import User
from models.user_session import UserSession
from models.base import Session
from werkzeug.security import generate_password_hash, check_password_hash
import re
from ..middleware.auth import auth_required # Corrected import path
import logging

logger = logging.getLogger(__name__)

bp = Blueprint('auth', __name__)  # Remove url_prefix

@bp.route("/login")
def login_page():
    if g.get('user'):
        return redirect(url_for('dashboard.dashboard'))
    
    # Check if any users exist
    session = Session()
    try:
        user_exists = session.query(User).first() is not None
        if not user_exists:
            return redirect(url_for('auth.register_page'))
        return render_template('login.html')
    finally:
        session.close()

@bp.route("/register")
def register_page():
    if g.get('user'):
        return redirect(url_for('dashboard.dashboard'))
    
    # Check if any users exist
    session = Session()
    try:
        user_exists = session.query(User).first() is not None
        if user_exists:
            return redirect(url_for('auth.login_page'))
        return render_template('register.html')
    finally:
        session.close()

@bp.route("/api/auth/register", methods=['POST'])
def register():
    try:
        data = request.json
        
        # Validate required fields
        required_fields = ['username', 'email', 'password', 'full_name']
        if not all(field in data for field in required_fields):
            return jsonify({"error": "Missing required fields"}), 400
        
        # Validate email format
        if not re.match(r"[^@]+@[^@]+\.[^@]+", data['email']):
            return jsonify({"error": "Invalid email format"}), 400
        
        # Validate username (alphanumeric and underscores only)
        if not re.match(r"^[a-zA-Z0-9_]+$", data['username']):
            return jsonify({"error": "Username can only contain letters, numbers, and underscores"}), 400
        
        session = Session()
        try:
            # Check if this is the first user
            is_first_user = session.query(User).first() is None
            
            # Check if username or email already exists
            if session.query(User).filter(User.username == data['username']).first():
                return jsonify({"error": "Username already taken"}), 409
            if session.query(User).filter(User.email == data['email']).first():
                return jsonify({"error": "Email already registered"}), 409
            
            # Create new user
            user = User(
                username=data['username'],
                email=data['email'],
                full_name=data['full_name'],
                role='admin' if is_first_user else 'user'  # First user is admin
            )
            user.set_password(data['password'])
            
            session.add(user)
            session.commit()
            
            return jsonify({
                "message": "User registered successfully",
                "user_id": user.id
            })
        finally:
            session.close()
            
    except Exception as e:
        logger.error(f"Registration error: {str(e)}")
        return jsonify({"error": "Registration failed"}), 500

@bp.route("/api/auth/login", methods=['POST'])
def login():
    try:
        data = request.json
        if not data:
            return jsonify({"error": "No data provided"}), 400
            
        username = data.get('username')
        password = data.get('password')
        
        if not username or not password:
            return jsonify({"error": "Username and password are required"}), 400
        
        user = User.get_by_username(username)
        if not user:
            return jsonify({"error": "Invalid username or password"}), 401
        
        if not user.check_password(password):
            return jsonify({"error": "Invalid username or password"}), 401
        
        if not user.is_active:
            return jsonify({"error": "Account is disabled"}), 403
        
        # Create session
        session_token = UserSession.create_session(
            user.id,
            request.remote_addr,
            request.user_agent.string
        )
        
        # Update last login
        user.update_last_login()
        
        response = jsonify({
            "message": "Login successful",
            "user": {
                "id": user.id,
                "username": user.username,
                "email": user.email,
                "full_name": user.full_name,
                "role": user.role
            }
        })
        
        # Set secure cookie
        response.set_cookie(
            'session_token',
            session_token,
            httponly=True,
            secure=False,  # Set to False for development, True for production
            samesite='Lax',
            max_age=30 * 24 * 60 * 60  # 30 days
        )
        
        return response
        
    except Exception as e:
        logger.error(f"Login error: {str(e)}", exc_info=True)
        return jsonify({"error": "Login failed. Please try again."}), 500

@bp.route("/api/auth/logout", methods=['POST'])
@auth_required()
def logout():
    try:
        auth_token = request.cookies.get('session_token')
        UserSession.invalidate_session(auth_token)
        
        response = jsonify({"message": "Logout successful"})
        response.delete_cookie('session_token')
        return response
        
    except Exception as e:
        logger.error(f"Logout error: {str(e)}")
        return jsonify({"error": "Logout failed"}), 500

@bp.route("/api/users/me", methods=['GET'])
@auth_required()
def get_current_user():
    return jsonify({
        "id": g.user.id,
        "username": g.user.username,
        "email": g.user.email,
        "full_name": g.user.full_name,
        "role": g.user.role,
        "image_url": g.user.image_url,
        "last_login": g.user.last_login.isoformat() if g.user.last_login else None
    })

@bp.route("/api/users/me", methods=['PUT'])
@auth_required()
def update_current_user():
    try:
        data = request.json
        session = Session()
        
        try:
            user = session.query(User).get(g.user.id)
            
            # Update allowed fields
            if 'full_name' in data:
                user.full_name = data['full_name']
            if 'email' in data:
                if User.get_by_email(data['email']) and data['email'] != user.email:
                    return jsonify({"error": "Email already registered"}), 409
                user.email = data['email']
            if 'image_url' in data:
                user.image_url = data['image_url']
            
            session.commit()
            
            return jsonify({
                "message": "Profile updated successfully",
                "user": {
                    "id": user.id,
                    "username": user.username,
                    "email": user.email,
                    "full_name": user.full_name,
                    "image_url": user.image_url
                }
            })
        finally:
            session.close()
            
    except Exception as e:
        logger.error(f"Profile update error: {str(e)}")
        return jsonify({"error": "Failed to update profile"}), 500

@bp.route("/api/users/me/password", methods=['PUT'])
@auth_required()
def change_password():
    try:
        data = request.json
        current_password = data.get('current_password')
        new_password = data.get('new_password')
        
        if not current_password or not new_password:
            return jsonify({"error": "Missing required fields"}), 400
        
        if not g.user.check_password(current_password):
            return jsonify({"error": "Current password is incorrect"}), 401
        
        session = Session()
        try:
            user = session.query(User).get(g.user.id)
            user.set_password(new_password)
            session.commit()
            
            # Invalidate all sessions
            user_sessions = session.query(UserSession).filter_by(user_id=user.id).all()
            for user_session in user_sessions:
                session.delete(user_session)
            session.commit()
            
            return jsonify({"message": "Password changed successfully"})
        finally:
            session.close()
            
    except Exception as e:
        logger.error(f"Password change error: {str(e)}")
        return jsonify({"error": "Failed to change password"}), 500

# Admin endpoints
@bp.route("/api/admin/users", methods=['GET'])
@auth_required(roles=['admin'])
def list_users():
    try:
        session = Session()
        try:
            users = session.query(User).all()
            return jsonify({
                "users": [{
                    "id": user.id,
                    "username": user.username,
                    "email": user.email,
                    "full_name": user.full_name,
                    "role": user.role,
                    "is_active": user.is_active,
                    "created_at": user.created_at.isoformat(),
                    "last_login": user.last_login.isoformat() if user.last_login else None
                } for user in users]
            })
        finally:
            session.close()
            
    except Exception as e:
        logger.error(f"List users error: {str(e)}")
        return jsonify({"error": "Failed to list users"}), 500

@bp.route("/api/admin/users/<int:user_id>", methods=['PUT'])
@auth_required(roles=['admin'])
def update_user(user_id):
    try:
        data = request.json
        session = Session()
        
        try:
            user = session.query(User).get(user_id)
            if not user:
                return jsonify({"error": "User not found"}), 404
            
            # Update allowed fields
            if 'role' in data:
                user.role = data['role']
            if 'is_active' in data:
                user.is_active = data['is_active']
                
                # If deactivating, invalidate all sessions
                if not data['is_active']:
                    user_sessions = session.query(UserSession).filter_by(user_id=user.id).all()
                    for user_session in user_sessions:
                        session.delete(user_session)
            
            session.commit()
            return jsonify({"message": "User updated successfully"})
        finally:
            session.close()
            
    except Exception as e:
        logger.error(f"Update user error: {str(e)}")
        return jsonify({"error": "Failed to update user"}), 500
